#include "komponentwidget.h"

#include <kprocess.h>
#include <kdebug.h>

#include <qtimer.h>

#include <cstdlib> // setenv

KomponentWidget::KomponentWidget(bool embedded, QWidget *parent, const char *name)
: QWidget(parent, name)
{
	setBackgroundColor(QColor(0, 0, 180));
	setMouseTracking(true);

	setFixedSize(800, 600);
	setCaption("PyroKomponent");
	show();

#ifdef EMBEDDED
	pxDict = NULL;
	setenv("PYTHONPATH", ".", 1);
	Py_Initialize();
#endif

	if(!embedded)
		QTimer::singleShot(500, this, SLOT(slotStarted()));
}

KomponentWidget::~KomponentWidget()
{
#ifdef EMBEDDED
	Py_Finalize();
#endif
}

void KomponentWidget::slotStarted()
{
	launchProgram();
}

void KomponentWidget::launchProgram()
{
#ifdef EMBEDDED
	QStringList args;
	PyObject *pName, *pModule;

	setenv("SDL_WINDOWID", QString("%1").arg(winId()).latin1(), 1);

	kdDebug() << "launch pyromaniac on window " << winId() << endl;

	pName = PyString_FromString("pyromaniac");
	pModule = PyImport_Import(pName);

	if(pModule) pxDict = PyModule_GetDict(pModule);
	else
	{
		kdDebug() << "dictionary for embedded program not found" << endl;
		return;
	}

	if(pModule) Py_DECREF(pModule);
	Py_DECREF(pName);

	execute("kickstart", args);
#else
	KProcess *proc = new KProcess();
	*proc << "pyromaniac.py" << "--hosted" << m_file.latin1();
	proc->setEnvironment("SDL_WINDOWID", QString("%1").arg(winId()));

	kdDebug() << "SDL_WINDOWID=" << winId() << endl;
	
	proc->start();
#endif
}

void KomponentWidget::execute(QString method, QStringList args)
{
#ifdef EMBEDDED
	PyObject *pxFunc;
	PyObject *pArgs, *pValue;
	QStringList::iterator it;
	int i;

	if(!pxDict) return;

	pxFunc = PyDict_GetItemString(pxDict, (char*)method.latin1());
	if(!pxFunc || !PyCallable_Check(pxFunc))
	{
		kdDebug() << "method " + method + " not found" << endl;
		return;
	}

	pArgs = PyTuple_New(args.count());
	i = 0;
	for(it = args.begin(); it != args.end(); it++)
	{
		kdDebug() << "arg " << i << " out of " << args.count() << " set to " << (*it) << endl;
		PyTuple_SetItem(pArgs, i, PyInt_FromLong((*it).toInt()));
		i++;
	}

	pValue = PyObject_CallObject(pxFunc, pArgs);
	if(pValue)
	{
		kdDebug() << "result is " << PyInt_AsLong(pValue) << endl;
		Py_DECREF(pValue);
	}

	Py_DECREF(pArgs);
#endif
}

void KomponentWidget::open(QString file)
{
	m_file = file;

	QTimer::singleShot(500, this, SLOT(launchProgram()));
}

void KomponentWidget::keyPressEvent(QKeyEvent *e)
{
	QStringList args;
	QString key, shift, alt;
	int keycode;

	kdDebug() << "key pressed: key: " << e->key() << " ascii: " << e->ascii() << " state: " << e->state() << endl;
	kdDebug() << "need: " << Qt::Key_Left << endl;

	keycode = e->ascii();
	switch(e->key())
	{
		case Qt::Key_Left:
			keycode = 276;
			break;
		case Qt::Key_Right:
			keycode = 275;
			break;
		case Qt::Key_Up:
			keycode = 273;
			break;
		case Qt::Key_Down:
			keycode = 274;
			break;
		case Qt::Key_PageUp:
			keycode = 280;
			break;
		case Qt::Key_PageDown:
			keycode = 281;
			break;
	}
	if((keycode >= 'A') && (keycode <= 'Z'))
	{
		if(e->state() & Qt::ShiftButton) keycode += 32;
	}

	key.setNum(keycode);
	shift.setNum(e->state() & Qt::ShiftButton);
	alt.setNum(e->state() & Qt::AltButton);
	args << key << shift << alt;

	execute("keyevent", args);
}

void KomponentWidget::mouseMoveEvent(QMouseEvent *e)
{
	QStringList args;
	QString x, y, clicked;

	kdDebug() << "mouse moved: " << e->x() << "," << e->y() << endl;

	x.setNum(e->x());
	y.setNum(e->y());
	clicked.setNum(0);
	args << x << y << clicked;

	execute("mouseevent", args);
}

void KomponentWidget::mousePressEvent(QMouseEvent *e)
{
	QStringList args;
	QString x, y, clicked;

	kdDebug() << "mouse pressed: " << e->x() << "," << e->y() << endl;

	x.setNum(e->x());
	y.setNum(e->y());
	clicked.setNum(1);
	args << x << y << clicked;

	execute("mouseevent", args);
}

void KomponentWidget::paintEvent(QPaintEvent *e)
{
	QStringList args;

	kdDebug() << "repaint" << endl;

	execute("redrawevent", args);
}

